/*
 * Entagged Audio Tag library
 * Copyright (c) 2003-2005 Raphal Slinckx <raphael@slinckx.net>
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *  
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */
package entagged.audioformats.mp3.util;

import entagged.audioformats.mp3.*;
import entagged.audioformats.mp3.util.id3frames.*;

import java.io.UnsupportedEncodingException;
import java.nio.*;
import java.util.*;

public class Id3v23TagReader {
	
	private Hashtable conversion;
	public Id3v23TagReader() {
		initConversionTable();
	}
	
	public Id3v2Tag read(ByteBuffer data, boolean[] ID3Flags, byte version) throws UnsupportedEncodingException {
		int tagSize = data.limit();
		byte[] b;
		Id3v2Tag tag = new Id3v2Tag();
		//----------------------------------------------------------------------------
		//Traitement en cas d'Header Etendu==true (A COMPLETER)
		if (version == Id3v2Tag.ID3V23 && ID3Flags[1])
			processExtendedHeader(data);
		//----------------------------------------------------------------------------
		//Extraction des champs de texte
		int specSize = (version == Id3v2Tag.ID3V22) ? 3 : 4;
		for (int a = 0; a < tagSize; a++) {
			//Nom de la Frame
			b = new byte[specSize];
			
			if(data.remaining() <= specSize)
				break;
			
			data.get(b);
			
			String field = new String(b);
			if (b[0] == 0)
				break;
			
			//La longueur du texte contenu dans la Frame
			int frameSize = readInteger(data, version);

			if ((frameSize > data.remaining()) || frameSize <= 0){
//				ignore empty frames
				System.err.println(field+" Frame size error, skiping the rest of the tag:"+frameSize);
				break;
			}
			
			b = new byte[ frameSize + ((version == Id3v2Tag.ID3V23) ? 2 : 0) ];
			data.get(b);
			
			if( ! "".equals(field)) {
				Id3Frame f = createId3Frame(field, b, version);
				if(f != null)
				    tag.add(f);
			}
		}
		
		return tag;
	}
	
	private void initConversionTable() {

		//TODO: APIC frame must update the mime-type to be converted ??
		//TODO: LINK frame (2.3) has a frame ID of 3-bytes making it incompatible with 2.3 frame ID of 4bytes, WTF???
		
		this.conversion = new Hashtable(100);
		String[] v22 = {
				"BUF", "CNT", "COM", "CRA",
				"CRM", "ETC", "EQU", "GEO",
				"IPL", "LNK", "MCI", "MLL",
				"PIC", "POP", "REV", "RVA",
				"SLT", "STC", "TAL", "TBP",
				"TCM", "TCO", "TCR", "TDA",
				"TDY", "TEN","TFT", "TIM",
				"TKE", "TLA", "TLE", "TMT",
				"TOA", "TOF", "TOL", "TOR",
				"TOT", "TP1", "TP2", "TP3",
				"TP4", "TPA", "TPB", "TRC",
				"TRD", "TRK", "TSI", "TSS",
				"TT1", "TT2", "TT3", "TXT",
				"TXX", "TYE", "UFI", "ULT",
				"WAF", "WAR", "WAS", "WCM",
				"WCP", "WPB", "WXX"
			};
		String[] v23 = {
				"RBUF", "PCNT", "COMM", "AENC",
				"", "ETCO", "EQUA", "GEOB",
				"IPLS", "LINK", "MCDI", "MLLT",
				"APIC", "POPM", "RVRB", "RVAD",
				"SYLT", "SYTC", "TALB", "TBPM",
				"TCOM", "TCON", "TCOP", "TDAT",
				"TDLY", "TENC", "TFLT", "TIME",
				"TKEY", "TLAN", "TLEN", "TMED",
				"TOPE", "TOFN", "TOLY", "TORY",
				"TOAL", "TPE1", "TPE2", "TPE3",
				"TPE4", "TPOS", "TPUB", "TSRC",
				"TRDA", "TRCK", "TSIZ", "TSSE",
				"TIT1", "TIT2", "TIT3", "TEXT",
				"TXXX", "TYER", "UFID", "USLT",
				"WOAF", "WOAR", "WOAS", "WCOM",
				"WCOP", "WPUB", "WXXX"			
			};
		
		for(int i = 0; i<v22.length; i++)
			this.conversion.put(v22[i], v23[i]);
	}
	
	private Id3Frame createId3Frame(String field, byte[] data, byte version) throws UnsupportedEncodingException {
		if(version == Id3v2Tag.ID3V22)
			field = convertFromId3v22(field);
		
		if("".equals(field))
			return null;
			
		//Text frames
		if (field.startsWith("T") && !field.startsWith("TX")) {
			return new TextId3Frame(field, data, version);
		}
		//Comment
		else if (field.startsWith("COMM"))
			return new CommId3Frame(data, version);
		//Universal file id
		else if (field.startsWith("UFID"))
			return new UfidId3Frame(data, version);
		else if (field.startsWith("APIC"))
		    return new ApicId3Frame(data, version);
		//Any other frame
		else
			return new GenericId3Frame(field, data, version);
	}
	
	private String convertFromId3v22(String field) {
		String s = (String) this.conversion.get(field);
		
		if(s == null)
			return "";
		
		return s;
	}

	//Process the Extended Header in the ID3v2 Tag, returns the number of bytes to skip
	private int processExtendedHeader(ByteBuffer data) {
		//TODO Verify that we have an syncsfe int
		byte[] exthead = new byte [4];
		data.get(exthead);
		int extsize = readInteger(data, Id3v2Tag.ID3V23);
		// The extended header size includes those first four bytes.
		data.position(data.position() + (extsize));
		return extsize;
	}

	private int readInteger(ByteBuffer bb, int version) {
		int value = 0;

		if(version == Id3v2Tag.ID3V23)
			value += (bb.get()& 0xFF) << 24;
		value += (bb.get()& 0xFF) << 16;
		value += (bb.get()& 0xFF) << 8;
		value += (bb.get()& 0xFF);

		return value;
	}
}
